﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.IO;
using System.IO.IsolatedStorage;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Interactivity;
using Infragistics.Controls.Grids;
using Infragistics.Persistence;
using Infragistics.Persistence.Primitives;

namespace FilteringXamGrid.Behaviors
{
    public class XamGridPersistenceBehavior : Behavior<XamGrid>
    {

        //private PersistenceGroup _persistenceGroup = new PersistenceGroup();
        private XamGrid Grid
        {
            get { return AssociatedObject; }
        }

        protected override void OnAttached()
        {
            base.OnAttached();
            SetSettings();
            Grid.Loaded += Grid_Loaded;
            Grid.Unloaded += Grid_Unloaded;
            Grid.Dispatcher.ShutdownStarted += Dispatcher_ShutdownStarted;
        }

        void Dispatcher_ShutdownStarted(object sender, EventArgs e)
        {
            SaveIntoIsolatedStorage(Grid.Name);
        }

        protected override void OnDetaching()
        {

            Grid.Loaded -= Grid_Loaded;
            Grid.Unloaded -= Grid_Unloaded;
            Grid.Dispatcher.ShutdownStarted -= Dispatcher_ShutdownStarted;
            base.OnDetaching();
        }

        private void Grid_Unloaded(object sender, RoutedEventArgs e)
        {
            SaveIntoIsolatedStorage(Grid.Name);
            Detach();
        }
        private void SaveIntoIsolatedStorage(string fileName)
        {
            foreach (Column c in Grid.Columns)
            {
                var i = Grid.Columns.IndexOf(c);
                c.Tag = i.ToString(CultureInfo.InvariantCulture);
            }

            // Save DependencyObject properties into MemoryStream            
            MemoryStream memoryStream = PersistenceManager.Save(Grid);
            // Obtains user-scoped isolated storage and creates a file with persistence settings            
            using (IsolatedStorageFile iso = IsolatedStorageFile.GetUserStoreForAssembly())
            {
                if (iso.FileExists(fileName))
                    iso.DeleteFile(fileName);
                using (var stream = new IsolatedStorageFileStream(fileName, FileMode.OpenOrCreate, iso))
                {
                    stream.Write(memoryStream.ToArray(), 0, (int)memoryStream.Length);
                }
            }
        }

        private void LoadFromIsolatedStorage(string fileName)
        {

            using (IsolatedStorageFile iso = IsolatedStorageFile.GetUserStoreForAssembly())
            {
                if (iso.FileExists(fileName))
                {
                    using (var stream = new IsolatedStorageFileStream(fileName, FileMode.Open, iso))
                    {
                        // loads persistence settings from IsolatedStorageFileStream into DependencyObject                        
                        PersistenceManager.Load(Grid, stream);
                    }
                }
            }
        }

        private void Grid_Loaded(object sender, RoutedEventArgs e)
        {

            LoadFromIsolatedStorage(Grid.Name);

        }

        private void SetSettings()
        {
            var persistenceSettings = new PersistenceSettings { SavePersistenceOptions = PersistenceOption.OnlySpecified, LoadPersistenceOptions = PersistenceOption.OnlySpecified };

  
            var pnpi = new PropertyNamePersistenceInfo { PropertyName = "Width" };
            persistenceSettings.PropertySettings.Add(pnpi);
            pnpi = new PropertyNamePersistenceInfo { PropertyName = "Visibility" };
            persistenceSettings.PropertySettings.Add(pnpi);

            PersistenceManager.SetSettings(Grid, persistenceSettings);


        }


    }
}
